/*
 *
 * Created on Oct 16, 2006, 2:03 PM
 */

package gov.bnl.gums.service;

import gov.bnl.gums.configuration.ConfigElement;
import gov.bnl.gums.configuration.Configuration;
import gov.bnl.gums.groupToAccount.*;
import gov.bnl.gums.hostToGroup.*;
import gov.bnl.gums.util.StringUtil;

import javax.servlet.http.HttpServletRequest;

import java.lang.reflect.Method;
import java.rmi.Remote;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;

/** 
 * Toolkit for providing configuration functionality for the web pages.
 * @author Jay Packard
 */
public class ConfigurationWebToolkit implements Remote {
	static Pattern configParamPattern = Pattern.compile("^ce_.*");
	
	static public Configuration removeUncheckedG2AMappings(HttpServletRequest request, Configuration configuration) {
		int i = 0;

		Configuration newConfiguration = (Configuration)configuration.clone();
		
		for (Object obj : configuration.getGroupToAccountMappings().values()) {
			GroupToAccountMapping g2aMapping = (GroupToAccountMapping)obj;
			if (request.getParameter("g"+i)==null)
				newConfiguration.removeGroupToAccountMapping(g2aMapping.getName(), false);
			i++;
		}
		
		return newConfiguration;
	}

	@SuppressWarnings("unchecked")
	static public ConfigElement parseConfigElement(HttpServletRequest request) throws Exception {
		String className = request.getParameter("class").trim();
		Class<ConfigElement> configElementClass = (Class<ConfigElement>)Class.forName(className);
		ConfigElement configElement = configElementClass.newInstance();
		
		Set<?> keySet = request.getParameterMap().keySet();
		Iterator<?> it = keySet.iterator();
		while(it.hasNext()) {
			String htmlKey = it.next().toString();
			if (!configParamPattern.matcher(htmlKey).matches())
				continue;
			String key = htmlKey.substring(3); // get past 'ce_' part
			String capFirstLetter = key.substring(0,1).toUpperCase();
			String setterName = "set" + capFirstLetter + key.substring(1, key.length());
			Method method = null;
			try {
				method = configElementClass.getMethod(setterName, String.class);
			} catch (Exception e) {
				continue;
			}
			
			assert(method.getParameterTypes().length == 1);
			Class paramClassType = method.getParameterTypes()[0];
			
			if (paramClassType.equals(String.class))
				method.invoke(configElement, request.getParameter(htmlKey));
			else if (paramClassType.equals(Boolean.class) || paramClassType.equals(boolean.class))
				method.invoke(configElement, Boolean.parseBoolean(request.getParameter(htmlKey)));
			else if (paramClassType.newInstance() instanceof List) {
				List<String> list = new ArrayList<String>();
				String strParam;
				int i = 0;
				while ((strParam = request.getParameter(key + new Integer(i++).toString())) != null)
					list.add(strParam);
				if (i==0) {
					// look for just starting letter of key instead of full key name
					String newKey = key.substring(0,1);
					while ((strParam = request.getParameter(newKey + new Integer(i++).toString())) != null)
						list.add(strParam);
				}
				method.invoke(configElement, list);
			}
			else if (paramClassType.newInstance() instanceof Set) {
				Set<String> list = new HashSet<String>();
				String strParam;
				int i = 0;
				while ((strParam = request.getParameter(key + new Integer(i).toString())) != null)
					list.add(strParam);
				if (i==0) {
					// look for just starting two letters of key instead of full key name
					String conciseKey = key.substring(0,2);
					while ((strParam = request.getParameter(conciseKey + new Integer(i++).toString())) != null)
						list.add(strParam);
				}
				method.invoke(configElement, list);
			}
		}
		
		return configElement;
	}		

	static public String getHostToGroupReferences(Configuration configuration, String g2AMappingName) {
		String retStr = null;
		Collection<HostToGroupMapping> h2GMappings = configuration.getHostToGroupMappings();
		Iterator<HostToGroupMapping> it = h2GMappings.iterator();
		while (it.hasNext()) {
			HostToGroupMapping h2GMapping = (HostToGroupMapping)it.next();
			Iterator<String> it2 = StringUtil.toList(h2GMapping.getGroupToAccountMappings()).iterator();
			while (it2.hasNext()) {
				String thisG2AMapping = (String)it2.next();
				if (thisG2AMapping.equals(g2AMappingName)) {
					if (retStr==null) 
						retStr = "";
					retStr += "\"" + h2GMapping.getName() + "\", ";
					break;
				}
			}
		}
		if(retStr!=null)
			retStr = retStr.substring(0, retStr.length()-2);
		return retStr;
	}

	static public String getGroupToAccountMappingReferences(Configuration configuration, String name, String className) {
		String retStr = null;
		Collection<GroupToAccountMapping> g2AMappings = configuration.getGroupToAccountMappings().values();
		Iterator<GroupToAccountMapping> it = g2AMappings.iterator();
		while (it.hasNext()) {
			GroupToAccountMapping g2AMapping = (GroupToAccountMapping)it.next();
			if(className.equals("gov.bnl.gums.account.AccountMapper")) {
				Iterator<String> it2 = StringUtil.toList(g2AMapping.getAccountMappers()).iterator();
				while (it2.hasNext()) {
					String thisAccountMapper = (String)it2.next().trim();
					if (thisAccountMapper.equals(name)) {
						if (retStr==null) 
							retStr = "";
						retStr += g2AMapping.getName() + ", ";
						break;
					}
				}
			}
			else if(className.equals("gov.bnl.gums.userGroup.UserGroup")) {
				Iterator<String> it2 = StringUtil.toList(g2AMapping.getUserGroups()).iterator();
				while (it2.hasNext()) {
					String thisUserGroup = (String)it2.next();
					if (thisUserGroup.equals(name)) {
						if (retStr==null) 
							retStr = "";
						retStr += g2AMapping.getName() + ", ";
						break;
					}
				}
			}
		}
		if(retStr!=null)
			retStr = retStr.substring(0, retStr.length()-2);
		return retStr;
	}

	static public String createSelectBox(String name, Collection<?> items, String selected, String javascript, boolean includeEmptySlot) {
		String retStr = "<select name=\""+name+"\" " + (javascript!=null?javascript:"") + ">";
		if (includeEmptySlot)
			retStr += "<option " + (selected==null?"selected":"") + "></option>";
		Iterator<?> it = items.iterator();
		while(it.hasNext())
		{
			Object item = it.next();
			String strVal;
			if (item instanceof ConfigElement)
				strVal = ((ConfigElement)item).getName();
			else if (item instanceof Class)
				strVal = ((Class<?>)item).getName();
			else
				strVal = item.toString();
			if (strVal.equals("_test") || strVal.equals("/DC=com/DC=example/OU=Services/CN=example.site.com"))
				continue;
			if (strVal.equals(selected))
				retStr += "<option selected>" + strVal + "</option>";
			else
				retStr += "<option>" + strVal + "</option>";
		}
		retStr += "</select> \n";
		return retStr;
	}
	
	static public Object blankNull(Object obj) {
		return (obj == null) ? "" : obj;
	}
	
	static public String createOrderedList(String name, Collection<?> items, String javascript) {
		// TODO - has to have sophisticated logic for moving position of each element in the ordered list
		return "";
	}

	static public String createDoSubmit(Collection<ConfigElement> items, HttpServletRequest request) {
		String str = 
			"<script language=\"javascript\">"+
			"String.prototype.trim = function() { return this.replace(/^\\s+|\\s+$/g, \"\"); };"+
			"document.forms[0].elements['ce_name'].value = document.forms[0].elements['ce_name'].value.trim();"+
			"function doSubmit(str) {";

		if ("add".equals(request.getParameter("command")) || "add".equals(request.getParameter("originalCommand"))) {
			str += "if ( document.forms[0].elements['ce_name'].value == '' ){ alert('First field cannot be empty'); return false; }";

			Iterator<ConfigElement> it = items.iterator();
			while(it.hasNext())
				str += "if ( document.forms[0].elements['ce_name'].value == '" + it.next().getName() + "'){ alert('Name already exists - please choose another name'); return false; }";
		}

		str += 
			"document.forms[0].elements['command'].value='save'; return true;"+
			"return false;"+
			"}"+
			"</script>";

		return str;
	}
	
	static public String stripVo(String voGroup) {
		if(voGroup.length()>0 && voGroup.charAt(0)=='/') {
			String subStr = voGroup.substring(1);
			int index = subStr.indexOf("/");
			if (index!=-1)
				return subStr.substring(index+1);
		}
		return voGroup;
	}

	static public String stripGroup(String voGroup) {
		if(voGroup.length()>0 && voGroup.charAt(0)=='/') {
			String subStr = voGroup.substring(1);
			int index = subStr.indexOf("/");
			if (index!=-1)
				return subStr.substring(0, index);
		}
		return voGroup;
	}	
}
